@@ -25,7 +25,7 @@
 /* XXX: recursive operations */
 
 #include "includes.h"
-RCSID("$OpenBSD: sftp-int.c,v 1.49 2002/09/12 00:13:06 djm Exp $");
+RCSID("$OpenBSD: sftp-int.c,v 1.57 2003/03/05 22:33:43 markus Exp $");
 
 #include <glob.h>
 
@@ -49,6 +49,9 @@ extern size_t copy_buffer_len;
 /* Number of concurrent outstanding requests */
 extern int num_requests;
 
+/* This is set to 0 if the progressmeter is not desired. */
+int showprogress = 1;
+
 /* Seperators for interactive commands */
 #define WHITESPACE " \t\r\n"
 
@@ -75,13 +78,14 @@ extern int num_requests;
 #define I_SHELL		20
 #define I_SYMLINK	21
 #define I_VERSION	22
+#define I_PROGRESS	23
 
 struct CMD {
 	const char *c;
 	const int n;
 };
 
-const struct CMD cmds[] = {
+static const struct CMD cmds[] = {
 	{ "bye",	I_QUIT },
 	{ "cd",		I_CHDIR },
 	{ "chdir",	I_CHDIR },
@@ -102,6 +106,7 @@ const struct CMD cmds[] = {
 	{ "ls",		I_LS },
 	{ "lumask",	I_LUMASK },
 	{ "mkdir",	I_MKDIR },
+	{ "progress",	I_PROGRESS },
 	{ "put",	I_PUT },
 	{ "mput",	I_PUT },
 	{ "pwd",	I_PWD },
@@ -134,6 +139,7 @@ help(void)
 	printf("ls [path]                     Display remote directory listing\n");
 	printf("lumask umask                  Set local umask to 'umask'\n");
 	printf("mkdir path                    Create remote directory\n");
+	printf("progress                      Toggle display of progress meter\n");
 	printf("put local-path [remote-path]  Upload file\n");
 	printf("pwd                           Display remote working directory\n");
 	printf("exit                          Quit sftp\n");
@@ -376,6 +382,17 @@ is_dir(char *path)
 	return(sb.st_mode & S_IFDIR);
 }
 
+static int
+is_reg(char *path)
+{
+	struct stat sb;
+
+	if (stat(path, &sb) == -1)
+		fatal("stat %s: %s", path, strerror(errno));
+
+	return(S_ISREG(sb.st_mode));
+}
+
 static int
 remote_is_dir(struct sftp_conn *conn, char *path)
 {
@@ -427,7 +444,6 @@ process_get(struct sftp_conn *conn, char *src, char *dst, char *pwd, int pflag)
 			err = -1;
 			goto out;
 		}
-		printf("Fetching %s to %s\n", g.gl_pathv[0], abs_dst);
 		err = do_download(conn, g.gl_pathv[0], abs_dst, pflag);
 		goto out;
 	}
@@ -491,6 +507,12 @@ process_put(struct sftp_conn *conn, char *src, char *dst, char *pwd, int pflag)
 
 	/* Only one match, dst may be file, directory or unspecified */
 	if (g.gl_pathv[0] && g.gl_matchc == 1) {
+		if (!is_reg(g.gl_pathv[0])) {
+			error("Can't upload %s: not a regular file",
+			    g.gl_pathv[0]);
+			err = 1;
+			goto out;
+		}
 		if (tmp_dst) {
 			/* If directory specified, append filename */
 			if (remote_is_dir(conn, tmp_dst)) {
@@ -509,7 +531,6 @@ process_put(struct sftp_conn *conn, char *src, char *dst, char *pwd, int pflag)
 			}
 			abs_dst = make_absolute(abs_dst, pwd);
 		}
-		printf("Uploading %s to %s\n", g.gl_pathv[0], abs_dst);
 		err = do_upload(conn, g.gl_pathv[0], abs_dst, pflag);
 		goto out;
 	}
@@ -523,6 +544,11 @@ process_put(struct sftp_conn *conn, char *src, char *dst, char *pwd, int pflag)
 	}
 
 	for (i = 0; g.gl_pathv[i]; i++) {
+		if (!is_reg(g.gl_pathv[i])) {
+			error("skipping non-regular file %s", 
+			    g.gl_pathv[i]);
+			continue;
+		}
 		if (infer_path(g.gl_pathv[i], &tmp)) {
 			err = -1;
 			goto out;
@@ -552,7 +578,7 @@ sdirent_comp(const void *aa, const void *bb)
 	SFTP_DIRENT *a = *(SFTP_DIRENT **)aa;
 	SFTP_DIRENT *b = *(SFTP_DIRENT **)bb;
 
-	return (strcmp(a->filename, b->filename));	
+	return (strcmp(a->filename, b->filename));
 }
 
 /* sftp ls.1 replacement for directories */
@@ -565,7 +591,7 @@ do_ls_dir(struct sftp_conn *conn, char *path, char *strip_path, int lflag)
 	if ((n = do_readdir(conn, path, &d)) != 0)
 		return (n);
 
-	/* Count entries for sort */	
+	/* Count entries for sort */
 	for (n = 0; d[n] != NULL; n++)
 		;
 
@@ -573,7 +599,7 @@ do_ls_dir(struct sftp_conn *conn, char *path, char *strip_path, int lflag)
 
 	for (n = 0; d[n] != NULL; n++) {
 		char *tmp, *fname;
-		
+
 		tmp = path_append(path, d[n]->filename);
 		fname = path_strip(tmp, strip_path);
 		xfree(tmp);
@@ -591,7 +617,7 @@ do_ls_dir(struct sftp_conn *conn, char *path, char *strip_path, int lflag)
 			/* XXX - multicolumn display would be nice here */
 			printf("%s\n", fname);
 		}
-		
+
 		xfree(fname);
 	}
 
@@ -601,7 +627,7 @@ do_ls_dir(struct sftp_conn *conn, char *path, char *strip_path, int lflag)
 
 /* sftp ls.1 replacement which handles path globs */
 static int
-do_globbed_ls(struct sftp_conn *conn, char *path, char *strip_path, 
+do_globbed_ls(struct sftp_conn *conn, char *path, char *strip_path,
     int lflag)
 {
 	glob_t g;
@@ -611,23 +637,23 @@ do_globbed_ls(struct sftp_conn *conn, char *path, char *strip_path,
 
 	memset(&g, 0, sizeof(g));
 
-	if (remote_glob(conn, path, GLOB_MARK|GLOB_NOCHECK|GLOB_BRACE, 
+	if (remote_glob(conn, path, GLOB_MARK|GLOB_NOCHECK|GLOB_BRACE,
 	    NULL, &g)) {
 		error("Can't ls: \"%s\" not found", path);
 		return (-1);
 	}
 
 	/*
-	 * If the glob returns a single match, which is the same as the 
+	 * If the glob returns a single match, which is the same as the
 	 * input glob, and it is a directory, then just list its contents
 	 */
-	if (g.gl_pathc == 1 && 
+	if (g.gl_pathc == 1 &&
 	    strncmp(path, g.gl_pathv[0], strlen(g.gl_pathv[0]) - 1) == 0) {
 		if ((a = do_lstat(conn, path, 1)) == NULL) {
 			globfree(&g);
 	    		return (-1);
 		}
-		if ((a->flags & SSH2_FILEXFER_ATTR_PERMISSIONS) && 
+		if ((a->flags & SSH2_FILEXFER_ATTR_PERMISSIONS) &&
 		    S_ISDIR(a->perm)) {
 			globfree(&g);
 			return (do_ls_dir(conn, path, strip_path, lflag));
@@ -642,8 +668,8 @@ do_globbed_ls(struct sftp_conn *conn, char *path, char *strip_path,
 		if (lflag) {
 			/*
 			 * XXX: this is slow - 1 roundtrip per path
-			 * A solution to this is to fork glob() and 
-			 * build a sftp specific version which keeps the 
+			 * A solution to this is to fork glob() and
+			 * build a sftp specific version which keeps the
 			 * attribs (which currently get thrown away)
 			 * that the server returns as well as the filenames.
 			 */
@@ -668,7 +694,7 @@ do_globbed_ls(struct sftp_conn *conn, char *path, char *strip_path,
 }
 
 static int
-parse_args(const char **cpp, int *pflag, int *lflag, 
+parse_args(const char **cpp, int *pflag, int *lflag, int *iflag,
     unsigned long *n_arg, char **path1, char **path2)
 {
 	const char *cmd, *cp = *cpp;
@@ -680,10 +706,17 @@ parse_args(const char **cpp, int *pflag, int *lflag,
 	/* Skip leading whitespace */
 	cp = cp + strspn(cp, WHITESPACE);
 
-	/* Ignore blank lines */
-	if (!*cp)
-		return(-1);
+	/* Ignore blank lines and lines which begin with comment '#' char */
+	if (*cp == '\0' || *cp == '#')
+		return (0);
 
+	/* Check for leading '-' (disable error processing) */
+	*iflag = 0;
+	if (*cp == '-') {
+		*iflag = 1;
+		cp++;
+	}
+		
 	/* Figure out which command we have */
 	for (i = 0; cmds[i].c; i++) {
 		int cmdlen = strlen(cmds[i].c);
@@ -705,7 +738,7 @@ parse_args(const char **cpp, int *pflag, int *lflag,
 		cmdnum = I_SHELL;
 	} else if (cmdnum == -1) {
 		error("Invalid command.");
-		return(-1);
+		return (-1);
 	}
 
 	/* Get arguments and parse flags */
@@ -805,6 +838,7 @@ parse_args(const char **cpp, int *pflag, int *lflag,
 	case I_LPWD:
 	case I_HELP:
 	case I_VERSION:
+	case I_PROGRESS:
 		break;
 	default:
 		fatal("Command not implemented");
@@ -815,10 +849,11 @@ parse_args(const char **cpp, int *pflag, int *lflag,
 }
 
 static int
-parse_dispatch_command(struct sftp_conn *conn, const char *cmd, char **pwd)
+parse_dispatch_command(struct sftp_conn *conn, const char *cmd, char **pwd,
+    int err_abort)
 {
 	char *path1, *path2, *tmp;
-	int pflag, lflag, cmdnum, i;
+	int pflag, lflag, iflag, cmdnum, i;
 	unsigned long n_arg;
 	Attrib a, *aa;
 	char path_buf[MAXPATHLEN];
@@ -826,14 +861,22 @@ parse_dispatch_command(struct sftp_conn *conn, const char *cmd, char **pwd)
 	glob_t g;
 
 	path1 = path2 = NULL;
-	cmdnum = parse_args(&cmd, &pflag, &lflag, &n_arg,
+	cmdnum = parse_args(&cmd, &pflag, &lflag, &iflag, &n_arg,
 	    &path1, &path2);
 
+	if (iflag != 0)
+		err_abort = 0;
+
 	memset(&g, 0, sizeof(g));
 
 	/* Perform command */
 	switch (cmdnum) {
+	case 0:
+		/* Blank line */
+		break;
 	case -1:
+		/* Unrecognized command */
+		err = -1;
 		break;
 	case I_GET:
 		err = process_get(conn, path1, path2, *pwd, pflag);
@@ -855,8 +898,9 @@ parse_dispatch_command(struct sftp_conn *conn, const char *cmd, char **pwd)
 		remote_glob(conn, path1, GLOB_NOCHECK, NULL, &g);
 		for (i = 0; g.gl_pathv[i]; i++) {
 			printf("Removing %s\n", g.gl_pathv[i]);
-			if (do_rm(conn, g.gl_pathv[i]) == -1)
-				err = -1;
+			err = do_rm(conn, g.gl_pathv[i]);
+			if (err != 0 && err_abort)
+				break;
 		}
 		break;
 	case I_MKDIR:
@@ -902,15 +946,14 @@ parse_dispatch_command(struct sftp_conn *conn, const char *cmd, char **pwd)
 			do_globbed_ls(conn, *pwd, *pwd, lflag);
 			break;
 		}
-		
+
 		/* Strip pwd off beginning of non-absolute paths */
 		tmp = NULL;
 		if (*path1 != '/')
 			tmp = *pwd;
 
 		path1 = make_absolute(path1, *pwd);
-
-		do_globbed_ls(conn, path1, tmp, lflag);
+		err = do_globbed_ls(conn, path1, tmp, lflag);
 		break;
 	case I_LCHDIR:
 		if (chdir(path1) == -1) {
@@ -944,62 +987,70 @@ parse_dispatch_command(struct sftp_conn *conn, const char *cmd, char **pwd)
 		remote_glob(conn, path1, GLOB_NOCHECK, NULL, &g);
 		for (i = 0; g.gl_pathv[i]; i++) {
 			printf("Changing mode on %s\n", g.gl_pathv[i]);
-			do_setstat(conn, g.gl_pathv[i], &a);
+			err = do_setstat(conn, g.gl_pathv[i], &a);
+			if (err != 0 && err_abort)
+				break;
 		}
 		break;
 	case I_CHOWN:
-		path1 = make_absolute(path1, *pwd);
-		remote_glob(conn, path1, GLOB_NOCHECK, NULL, &g);
-		for (i = 0; g.gl_pathv[i]; i++) {
-			if (!(aa = do_stat(conn, g.gl_pathv[i], 0)))
-				continue;
-			if (!(aa->flags & SSH2_FILEXFER_ATTR_UIDGID)) {
-				error("Can't get current ownership of "
-				    "remote file \"%s\"", g.gl_pathv[i]);
-				continue;
-			}
-			printf("Changing owner on %s\n", g.gl_pathv[i]);
-			aa->flags &= SSH2_FILEXFER_ATTR_UIDGID;
-			aa->uid = n_arg;
-			do_setstat(conn, g.gl_pathv[i], aa);
-		}
-		break;
 	case I_CHGRP:
 		path1 = make_absolute(path1, *pwd);
 		remote_glob(conn, path1, GLOB_NOCHECK, NULL, &g);
 		for (i = 0; g.gl_pathv[i]; i++) {
-			if (!(aa = do_stat(conn, g.gl_pathv[i], 0)))
-				continue;
+			if (!(aa = do_stat(conn, g.gl_pathv[i], 0))) {
+				if (err != 0 && err_abort)
+					break;
+				else
+					continue;
+			}
 			if (!(aa->flags & SSH2_FILEXFER_ATTR_UIDGID)) {
 				error("Can't get current ownership of "
 				    "remote file \"%s\"", g.gl_pathv[i]);
-				continue;
+				if (err != 0 && err_abort)
+					break;
+				else
+					continue;
 			}
-			printf("Changing group on %s\n", g.gl_pathv[i]);
 			aa->flags &= SSH2_FILEXFER_ATTR_UIDGID;
-			aa->gid = n_arg;
-			do_setstat(conn, g.gl_pathv[i], aa);
+			if (cmdnum == I_CHOWN) {
+				printf("Changing owner on %s\n", g.gl_pathv[i]);
+				aa->uid = n_arg;
+			} else {
+				printf("Changing group on %s\n", g.gl_pathv[i]);
+				aa->gid = n_arg;
+			}
+			err = do_setstat(conn, g.gl_pathv[i], aa);
+			if (err != 0 && err_abort)
+				break;
 		}
 		break;
 	case I_PWD:
 		printf("Remote working directory: %s\n", *pwd);
 		break;
 	case I_LPWD:
-		if (!getcwd(path_buf, sizeof(path_buf)))
-			error("Couldn't get local cwd: %s",
-			    strerror(errno));
-		else
-			printf("Local working directory: %s\n",
-			    path_buf);
+		if (!getcwd(path_buf, sizeof(path_buf))) {
+			error("Couldn't get local cwd: %s", strerror(errno));
+			err = -1;
+			break;
+		}
+		printf("Local working directory: %s\n", path_buf);
 		break;
 	case I_QUIT:
-		return(-1);
+		/* Processed below */
+		break;
 	case I_HELP:
 		help();
 		break;
 	case I_VERSION:
 		printf("SFTP protocol version %u\n", sftp_proto_version(conn));
 		break;
+	case I_PROGRESS:
+		showprogress = !showprogress;
+		if (showprogress)
+			printf("Progress meter enabled\n");
+		else
+			printf("Progress meter disabled\n");
+		break;
 	default:
 		fatal("%d is not implemented", cmdnum);
 	}
@@ -1011,20 +1062,23 @@ parse_dispatch_command(struct sftp_conn *conn, const char *cmd, char **pwd)
 	if (path2)
 		xfree(path2);
 
-	/* If an error occurs in batch mode we should abort. */
-	if (infile != stdin && err > 0)
-		return -1;
+	/* If an unignored error occurs in batch mode we should abort. */
+	if (err_abort && err != 0)
+		return (-1);
+	else if (cmdnum == I_QUIT)
+		return (1);
 
-	return(0);
+	return (0);
 }
 
-void
+int
 interactive_loop(int fd_in, int fd_out, char *file1, char *file2)
 {
 	char *pwd;
 	char *dir = NULL;
 	char cmd[2048];
 	struct sftp_conn *conn;
+	int err;
 
 	conn = do_init(fd_in, fd_out, copy_buffer_len, num_requests);
 	if (conn == NULL)
@@ -1041,7 +1095,8 @@ interactive_loop(int fd_in, int fd_out, char *file1, char *file2)
 		if (remote_is_dir(conn, dir) && file2 == NULL) {
 			printf("Changing to: %s\n", dir);
 			snprintf(cmd, sizeof cmd, "cd \"%s\"", dir);
-			parse_dispatch_command(conn, cmd, &pwd);
+			if (parse_dispatch_command(conn, cmd, &pwd, 1) != 0)
+				return (-1);
 		} else {
 			if (file2 == NULL)
 				snprintf(cmd, sizeof cmd, "get %s", dir);
@@ -1049,15 +1104,18 @@ interactive_loop(int fd_in, int fd_out, char *file1, char *file2)
 				snprintf(cmd, sizeof cmd, "get %s %s", dir,
 				    file2);
 
-			parse_dispatch_command(conn, cmd, &pwd);
+			err = parse_dispatch_command(conn, cmd, &pwd, 1);
 			xfree(dir);
-			return;
+			xfree(pwd);
+			return (err);
 		}
 		xfree(dir);
 	}
+
 	setvbuf(stdout, NULL, _IOLBF, 0);
 	setvbuf(infile, NULL, _IOLBF, 0);
 
+	err = 0;
 	for (;;) {
 		char *cp;
 
@@ -1074,8 +1132,13 @@ interactive_loop(int fd_in, int fd_out, char *file1, char *file2)
 		if (cp)
 			*cp = '\0';
 
-		if (parse_dispatch_command(conn, cmd, &pwd))
+		err = parse_dispatch_command(conn, cmd, &pwd, infile != stdin);
+		if (err != 0)
 			break;
 	}
 	xfree(pwd);
+
+	/* err == 1 signifies normal "quit" exit */
+	return (err >= 0 ? 0 : -1);
 }
+
